define([
    'dojo/_base/declare',
    'dojo/_base/lang',
    'dojo/_base/window',
    'dijit/layout/ContentPane',
    'dijit/layout/TabContainer',
    'dojox/widget/Standby',
    '../../store',
    '../../widgets/Selector'
],function(declare,lang,win,ContentPane,TabContainer,Standby,store,Selector){
    return declare([TabContainer],{
        tabPosition: 'left-h',
        include: 'all',

        selectedActions: null,
        itemActions: null,
        dndType: "question",

        _dataMap: null,
        _busyCount: 0,
        constructor: function(){
            this.inherited(arguments);
            this._dataMap = {};
        },
        postCreate: function() {
            this.inherited(arguments);
            this._query = '_design/questions/_view/'+this.include;
        },
        startup: function() {
            if ( this._started ){ return; }
            this.inherited(arguments);
            this._busyWidget = new Standby({
                target: this.domNode,
                duration: 200
            }).placeAt(win.body());
            this._busyWidget.startup();
            this.watch("selectedChildWidget",lang.hitch(this,function(name,oldTab,newTab){
                this._fillCategoryTab(newTab.__category);
            }));
            store.query(this._query, {reduce:true,group:true,group_level:1})
            .forPairs(lang.hitch(this,function(value,key){
                this._createCategoryTab(key[0],value);
            }));
        },
        _createCategoryTab: function(category,count) {
            if (this._dataMap[category] === undefined) {
                var categoryTab = new ContentPane({
                    __category: category,
                    title: (category || '[No category]')+" ("+count+")"
                });
                categoryTab.startup();
                this._dataMap[category] = {
                    _widget: categoryTab
                };
                this.addChild(categoryTab);
            }
        },
        _fillCategoryTab: function(category) {
            var categoryMap = this._dataMap[category];
            if (!categoryMap._filled) {
                this._busy();
                categoryMap._filled = true;
                store.query(this._query, {reduce:true,group:true,group_level:2,startkey:[category],endkey:[category,{}]})
                .forPairs(lang.hitch(this,function(value,key){
                    this._createTopicSelector(key[1],category,value);
                })).then(lang.hitch(this,function(){
                    this._done();
                }));
            }
        },
        _createTopicSelector: function(topic,category,count){
            var categoryMap = this._dataMap[category];
            if (categoryMap[topic] === undefined) {
                var w = new Selector({
                    __category: category,
                    __topic: topic,
                    dndType: this.dndType,
                    title: (topic || '[No topic]')+" ("+count+")",
                    selectedActions: this.selectedActions,
                    itemActions: this.itemActions,
                    itemTitle: this._itemTitle
                }).placeAt(categoryMap._widget.containerNode);
                w.startup();
                categoryMap[topic] = {
                    _widget: w
                };
                this._fillTopicSelector(topic,category);
            }
        },
        _itemTitle: function(item) {
            var title = '['+item.code+']'+item.title;
            if ( this.include === 'all' ) {
                title += (item.publicationDate?' (published on '+item.publicationDate+')':' (unpublished)');
            }
            return title;
        },
        _fillTopicSelector: function(topic,category) {
            var categoryMap = this._dataMap[category];
            var topicMap = categoryMap[topic];
            if (!topicMap._filled) {
                topicMap._filled = true;
                this._busy();
                store.query(this._query, {
                    reduce:false,
                    include_docs:true,
                    key:[category,topic]
                }).forEach(lang.hitch(this,function(value){
                    topicMap._widget.addItem(value);
                })).then(lang.hitch(this,function(){
                    this._done();
                }));
            }
        },
        _busy: function() {
            if ( this._busyCount === 0 ) {
                this._busyWidget.show();
            }
            this._busyCount++;
        },
        _done: function() {
            if ( this._busyCount > 0 ) {
                this._busyCount--;
                if ( this._busyCount === 0 ) {
                    this._busyWidget.hide();
                }
            } else {
                console.warn('_done() was called more times than _busy().');
            }
        },
        destroy: function() {
            this._busyWidget.destroyRecursive();
            this.inherited(arguments);
        }
    });
});
