define([
    "../app/Content",
    "../app/Page",
    "../lib/async",
    "../model/classes/Response",
    "../model/classes/Survey",
    "../model/classes/SurveyRun",
    "../store",
    "dojo/_base/declare",
    "dojo/_base/event",
    "dojo/_base/json",
    "dojo/_base/lang",
    "dojo/promise/all",
    "dojo/request",
    "dojo/when",
    "require",
    "dojo/text!./templates/response.html"
], function(Content, Page, async, Response, Survey, SurveyRun, store, declare, event, json, lang, all, request, when, require, template) {
    return declare([Page],{
        contextRequire: require,
        templateString: template,
        response: null,
        postCreate: function(){
            this.options = this.options || {};
        },
        startup: function() {
            if ( this._started ) { return; }
            this.inherited(arguments);
            this._disableSubmit();
            var surveyRunId = this.surveyRunId;
            var responseId = this.options && this.options.id;
            if ( surveyRunId && responseId ) {
                this._loadSurveyAndResponse(surveyRunId,responseId)
                .then(lang.hitch(this, function() {
                    if ( this.response.publicationDate ) {
                        this._showInfo("<div>You already submitted your survey and cannot change it anymore. You can still view your answers here.</div>");
                        this._disableSubmit();
                    } else {
                        this._enableSubmit();
                    }
                }), lang.hitch(this,function() {
                    this._showInfo("<div>The url seems to be incorrect, no survey found.</div>");
                }));
            } else {
                throw new Error("No valid uid or survey passed!");
            }
        },
        _loadSurveyAndResponse: function(surveyRunId,responseId){
            return all([request.get('/api/surveyRuns/'+surveyRunId,{handleAs:'json'}),
                        request.get('/api/responses/'+responseId,{handleAs:'json'})])
            .then(lang.hitch(this,function(surveyAndResponse){
                var surveyRun = surveyAndResponse[0];
                this.response = surveyAndResponse[1];
                if ( this.response.surveyRunId !== surveyRunId ) {
                    throw "Survey does not match the response...";
                }
                this.titleNode.innerHTML = Survey.DisplayTitle.get(surveyRun.survey);
                this.surveyWidget.set('survey', surveyRun.survey);
                this.surveyWidget.set('value', this.response.answers || {});
            }));
        },
        _enableSubmit: function() {
            this.submitButton.set('disabled',false);
            this.continueButton.set('disabled',false);
            this.surveyWidget.set('disabled', false);
        },
        _disableSubmit: function() {
            this.submitButton.set('disabled',true);
            this.continueButton.set('disabled',true);
            this.surveyWidget.set('disabled', true);
        },
        _showInfo: function(html) {
            this.infoNode.innerHTML = html;
        },
        _getAnswersAndSaveResponse: function() {
            var answers = this.surveyWidget.get('value');
            this.response.answers = answers;
            return request.put('/api/responses/'+store.getIdentity(this.response),{
                handleAs:'json',
                data:json.toJson(this.response),
                headers:{"Content-Type": "application/json"}
            }).then(function(){
                Content.notify("Your response is saved.");
            }, function(err){
                Content.notify(err,'error');
            });
        },
        _onSubmit: function(e) {
            this.response.publicationDate = store.timestamp();
            this._getAnswersAndSaveResponse()
            .then(lang.hitch(this,function(){
                this._showInfo("<div>Thanks for filling in the survey. You cannot edit your answers anymore.</div>");
                this._disableSubmit();
            }));
            if ( e ) { event.stop(e); }
            return false;
        },
        _onContinueLater: function(e) {
            this._getAnswersAndSaveResponse()
            .then(lang.hitch(this,function(){
                this._showInfo("<div>To continue with this survey later, just save the URL in the location bar and revisit it later. Your answers will still be there.</div>");
            }));
            if ( e ) { event.stop(e); }
            return false;
        },
        _ignoreEvent: function(e) {
            if ( e ) { event.stop(e); }
            return false;
        }
    });
});
